package models

import (
	"gogs.mikescher.com/BlackForestBytes/goext/langext"
	"gogs.mikescher.com/BlackForestBytes/goext/rfctime"
)

type SolseitAPICall struct {
	FAPIRequestID SolseitAPICallID         `json:"id"                bson:"_id,omitempty"`
	Time          rfctime.RFC3339NanoTime  `json:"time"              bson:"time"`
	Success       bool                     `json:"success"           bson:"success"`
	ErrorStr      *string                  `json:"errorStr"          bson:"errorStr"`
	CallingMethod string                   `json:"callingMethod"     bson:"callingMethod"`
	Duration      rfctime.SecondsF64       `json:"duration"          bson:"duration"`
	Request       SolseitAPICallRequest    `json:"request"           bson:"request"`
	Response      *SolseitAPICallResponse  `json:"response"          bson:"response"`
	Deleted       *rfctime.RFC3339NanoTime `json:"deleted"           bson:"deleted"`
}

func (c SolseitAPICall) ToCrawlerHTTPCall() CrawlerLogHTTPCall {

	var resp *CrawlerHTTPResponse = nil
	if c.Response != nil {
		resp = &CrawlerHTTPResponse{
			ResponseStatus: c.Response.ResponseStatus,
			ResponseBody:   c.Response.ResponseBody,
			Header:         langext.ArrMap(c.Response.Header, func(v SolseitAPICallHeaderDef) CrawlerHTTPHeaderDef { return CrawlerHTTPHeaderDef{v.Key, v.Values} }),
			Message:        "",
		}
	}

	return CrawlerLogHTTPCall{
		Success:       c.Success,
		ErrorStr:      c.ErrorStr,
		CallingMethod: c.CallingMethod,
		Duration:      c.Duration,
		Request: CrawlerHTTPRequest{
			Method:      c.Request.Method,
			URL:         c.Request.URL,
			RequestBody: c.Request.RequestBody,
			Header:      langext.ArrMap(c.Request.Header, func(v SolseitAPICallHeaderDef) CrawlerHTTPHeaderDef { return CrawlerHTTPHeaderDef{v.Key, v.Values} }),
		},
		Response: resp,
	}
}

type SolseitAPICallRequest struct {
	Method      string                    `json:"method"            bson:"method"`
	URL         string                    `json:"url"               bson:"url"`
	RequestBody string                    `json:"requestBody"       bson:"requestBody"`
	Header      []SolseitAPICallHeaderDef `json:"header"            bson:"header"`
}

type SolseitAPICallResponse struct {
	ResponseStatus int                       `json:"responseStatus"     bson:"responseStatus"`
	ResponseBody   *string                   `json:"responseBody"       bson:"responseBody"`
	Header         []SolseitAPICallHeaderDef `json:"header"             bson:"header"`
}

type SolseitAPICallHeaderDef struct {
	Key    string
	Values []string
}
